classdef EllipseFunction1
    %FUNCLASS Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
%         Params = [1 4]';
        Params = [2 1 -1 -0.5]';
        Height;
        Width;
    end
    
    methods %Get set properties
        
        function obj = set.Height(obj,val)
            obj.Params(1) = val;
        end
        
        function x = get.Height(obj)
            x = obj.Params(1);
        end
        
        function obj = set.Width(obj,val)
            obj.Params(2) = val;
        end
        
        function x = get.Width(obj)
            x = obj.Params(2);
        end        
    end
    
    methods (Access = public)
        
        function pfixout = pfix(obj)
            pfixout =[
%                 0 obj.Height
%                 obj.Width 0
%                 0 -obj.Height
%                 -obj.Width 0
%                 obj.Params(3,1)  obj.Params(4,1)
%                 obj.Params(3,1)+0.05  obj.Params(4,1)
%                 obj.Params(3,1)+0.1  obj.Params(4,1)
                ];

        end
        
        function pboundsout = pbounds(obj)
            pboundsout =[-obj.Width -obj.Height 
                          obj.Width obj.Height];
        end
        
        function [Func] = fd(obj,p)
            %Func This returns the distance function
            % This returns signed distance function for a given point
            Func = sign((p(:,1)./obj.Width).^2 + (p(:,2)./obj.Height).^2 - 1).*(sum(obj.distance(p).^2,2)).^0.5;
        end
        
        function [Func] = F(obj,p)
            %Func This returns the distance function
            % This returns signed distance function for a given point
            Func = (p(:,1)./obj.Width).^2 + (p(:,2)./obj.Height).^2 - 1;
        end
        
        function [Grad] = dF(obj,p)
            %dF Summary of this method goes here
            %  This returnd the first order partial derivitves of the
            %  function for a given point
            dRdu1(:,1) = (2*p(:,1))./(obj.Width^2);
            dRdu2(:,1) = (2*p(:,2))./(obj.Height^2);
            Grad = [dRdu1 dRdu2];
        end
        
        function [SecondGrad] = ddF(obj,p)
            %ddF Second order partial derivitives (risidual derivitives)
            %  used for La Grange and CB2 Ellim approx
            
            ddRdu1du1 = 2/(obj.Width^2);
            ddRdu1du2 = 0;
            ddRdu2du1 = 0;
            ddRdu2du2 = 2/(obj.Height^2);
            
            SecondGrad = [ddRdu1du1 ddRdu1du2;
                          ddRdu2du1 ddRdu2du2];
        end
        
        function [DF] = dFelimActual(obj,p,slavedof)
            if slavedof == 1
                s = p(1,1); %x s 
                m = p(1,2);
                
                b = obj.Width;
                a = obj.Height; 
             
            elseif slavedof == 2
                
                s = p(1,2); %y slave
                m = p(1,1);
                
                a = obj.Width;
                b = obj.Height;
            end
            
            DF = -b^2*m/a^2/s;
            
        end
        
        function [SecondGrad] = ddFelimActual(obj,p,slavedof)
            %ddFelimActual Second order actual derivitives for ellimination
            %  used for La Grange and CB2 Ellim approx
                       
            if slavedof == 1
                s = p(1,1); %x s 
                m = p(1,2);
                
                b = obj.Width;
                a = obj.Height; 
             
            elseif slavedof == 2
                
                s = p(1,2); %y slave
                m = p(1,1);
                
                a = obj.Width;
                b = obj.Height;
            end
            
            SecondGrad = -b^4/a^2/s^3;
        end
        
        function [Supdate] = FuncFelimActual(obj,p,update,masterdof)
            %ddFelimActual Second order actual derivitives for ellimination
            %  used for La Grange and CB2 Ellim approx
                       
            
            if masterdof == 2
                %x s
                s = p(1,1);
                m = p(1,2);
                a = obj.Height; 
                b = obj.Width;
                
            elseif masterdof == 1
                s = p(1,2);
                m = p(1,1);
                a = obj.Width;
                b = obj.Height;
            end
            m = m + update;
            
            Supdate1 = b*(1-(m/a).^2)^0.5 - s;
            Supdate2 = -b*(1-(m/a).^2)^0.5 - s;
            
            if abs(Supdate1) < abs(Supdate2)
                Supdate = Supdate1;
            else
                Supdate = Supdate2;
            end 
        end
        
        function [pOut]= ClosestPoint(obj,p) %Gives point on closest boundary
            Np = size(p,1);
            pOut = zeros(size(p));
% %             for i = 1:Np
% %                 fx = @(t) obj.Width*cos(2*pi*t);
% %                 fy = @(t) obj.Height*sin(2*pi*t);
% %                 
% %                 f = @(x) (fx(x)-p(i,1))^2 + (fy(x)-p(i,2))^2;
% %                 
% %                 [t1, fval1] = fminsearch(f,0);
% %                 [t2, fval2] = fminsearch(f,0.3);
% %                 [t3, fval3] = fminsearch(f,0.6);
% %                 
% %                 if fval1 < fval2
% %                     t = t1;
% %                     fn = fval1;
% %                 else
% %                     t = t2;
% %                     fn = fval2;
% %                 end
% %                 
% %                 if fval3 < fn
% %                     t = t3;
% %                 end
% %                 
% %                 pOut(i,:) = [fx(t) fy(t)];
% %             end
               t = 0:1/2000:1;               
               fx =  obj.Width*cos(2*pi*t)';
               fy =  obj.Height*sin(2*pi*t)';
           for i = 1:Np
               
               dists = sum([fx-p(i,1), fy-p(i,2)].^2,2).^0.5;
               a = find(dists== min(dists));
               
               pOut(i,:)=[fx(a(1)) fy(a(1))];
           end

        end
         
        function [vecOut]= distance(obj,ps)
            %Gives vector to closest boundary point
            pboundary = obj.ClosestPoint(ps);
            vecOut = pboundary-ps;
        end
        
        function [pOut]= ClosestPoint2(obj,p) %Gives point on closest boundary
            Np = size(p,1);
            pOut = zeros(size(p));
            fx = @(t) obj.Width*cos(2*pi*t);
            fy = @(t) obj.Height*sin(2*pi*t);
            
            dfx = @(t) -obj.Width*sin(2*pi*t)*2*pi;
            dfy = @(t) obj.Height*cos(2*pi*t)*2*pi;
            
            for i = 1:Np
                
                f = @(x) ((fx(x)-p(i,1))^2 + (fy(x)-p(i,2))^2)^0.5;
                
                len = norm(p); 
                if (p(1,1) >= 0) && (p(1,2) >=0)
                    tstart = asin(p(1,2)/len)./2/pi;
                elseif (p(1,1) <= 0) && (p(1,2) >= 0)
                    tstart = 0.5 - asin(p(1,2)/len)./2/pi;
                elseif (p(1,1) <= 0) && (p(1,2) <=0)
                    tstart = 0.5 - asin(p(1,2)/len)./2/pi;
                else
                    tstart = 1 +  asin(p(1,2)/len)./2/pi;
                end
                
                t = fminsearch(f,tstart);
                
                pOut(i,:) = [fx(t) fy(t)];
            end
            
        end
       
        function [pOut]= ClosestPoint3(obj,p) %Gives point on closest boundary
            Np = size(p,1);
            pOut = zeros(size(p));
            fx = @(t) obj.Width*cos(2*pi*t);
            fy = @(t) obj.Height*sin(2*pi*t);
            
            dfx = @(t) -obj.Width*sin(2*pi*t)*2*pi;
            dfy = @(t) obj.Height*cos(2*pi*t)*2*pi;
            
            for i = 1:Np
                
                f = @(x) (fx(x)-p(i,1))^2 + (fy(x)-p(i,2))^2;
                g = @(x) (fx(x)-p(i,1))*2*dfx(x)* + (fy(x)-p(i,2))*2*dfy(x);
                fun = @(x) obj.myoptimfunction(f,g,x);
                len = norm(p); 
                if (p(1,1) >= 0) && (p(1,2) >=0)
                    tstart = asin(p(1,2)/len)./2/pi;
                elseif (p(1,1) <= 0) && (p(1,2) >= 0)
                    tstart = 0.5 - asin(p(1,2)/len)./2/pi;
                elseif (p(1,1) <= 0) && (p(1,2) <=0)
                    tstart = 0.5 - asin(p(1,2)/len)./2/pi;
                else
                    tstart = 1 +  asin(p(1,2)/len)./2/pi;
                end
                
                t = fminsearch(fun,tstart);
                
                pOut(i,:) = [fx(t) fy(t)];
            end
            
        end
        
        
        function xy = plot(obj)
            %plot Gives boudary plot points
            % This returns signed distance function for a given point
            
            x(:,1) = linspace(-obj.Width,obj.Width,1000);
            y1 = (-(x./obj.Width).^2 + 1).^0.5.*obj.Height;
            y2 = -(-(x./obj.Width).^2 + 1).^0.5*obj.Height;
            xy =[x, y1;
                flip(x), flip(y2)];
        end
        
        function [Grad] = tangent(obj,p)
            %dF Summary of this method goes here
            %  This returns the tangent at the point
            
            dRdu1(:,1) = (2*p(:,1))./obj.Width^2;
            dRdu2(:,1) = (2*p(:,2))./obj.Height^2;
            lclGrad = [dRdu1 dRdu2];
            Grad  = lclGrad./norm(lclGrad);
        end
        
    end
    
    methods (Access = private)
        
        function [F, G] = myoptimfunction(obj,f,g,t)
            %dF Summary of this method goes here
            %  This returnd the first order partial derivitves of the function
            F = f(t);
            G = g(t);
            
            
        end
        
        
    end
end

