classdef EllipseFunction4_3d
    %FUNCLASS Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        Xscale = 2; %x
        Yscale = 2; %y
        Zscale = 2; %z
    end
    
    methods (Access = public)
        
        function pfixout = pfix(obj)
            pfixout =[obj.Xscale 0 0
                -obj.Xscale 0 0
                0 obj.Yscale 0
                0 -obj.Yscale 0
                0 0 obj.Zscale
                0 0 -obj.Zscale];
        end
        
        function pboundsout = pbounds(obj)
            pboundsout =[-obj.Xscale -obj.Yscale -obj.Zscale
                obj.Xscale  obj.Yscale  obj.Zscale];
        end
        
        function [R] = fd(obj,p)
            %Func This returns the distance function
            % This returns signed distance function for a given point
            
            R =(p(:,1).^2.*obj.Yscale.^2.*obj.Zscale.^2 + obj.Xscale.^2.*p(:,2).^2.*obj.Zscale.^2 + obj.Xscale.^2.*obj.Yscale.^2.*p(:,3).^2).^(1./2) - obj.Xscale.*obj.Yscale.*obj.Zscale;

        end
        
        function [Grad] = dF(obj,p)
            %dF Summary of this method goes here
            %  This returnd the first order partial derivitves of the
            %  function for a given point
            dRdx =(p(:,1).*obj.Yscale.^2.*obj.Zscale.^2)./(p(:,1).^2.*obj.Yscale.^2.*obj.Zscale.^2 + obj.Xscale.^2.*p(:,2).^2.*obj.Zscale.^2 + obj.Xscale.^2.*obj.Yscale.^2.*p(:,3).^2).^(1./2);
            dRdy =(obj.Xscale.^2.*p(:,2).*obj.Zscale.^2)./(p(:,1).^2.*obj.Yscale.^2.*obj.Zscale.^2 + obj.Xscale.^2.*p(:,2).^2.*obj.Zscale.^2 + obj.Xscale.^2.*obj.Yscale.^2.*p(:,3).^2).^(1./2);
            dRdz =(obj.Xscale.^2.*obj.Yscale.^2.*p(:,3))./(p(:,1).^2.*obj.Yscale.^2.*obj.Zscale.^2 + obj.Xscale.^2.*p(:,2).^2.*obj.Zscale.^2 + obj.Xscale.^2.*obj.Yscale.^2.*p(:,3).^2).^(1./2);
            Grad = [dRdx dRdy dRdz];
        end
        
        function [SecondGrad] = ddF(obj,p)
            %ddF Second order partial derivitives (risidual derivitives)
            %  used for La Grange and CB2 Ellim approx
            
            ddRdxdx =(obj.Xscale^2*obj.Yscale^2*obj.Zscale^2*(p(:,2)^2*obj.Zscale^2 + obj.Yscale^2*p(:,3)^2))/(p(:,1)^2*obj.Yscale^2*obj.Zscale^2 + obj.Xscale^2*p(:,2)^2*obj.Zscale^2 + obj.Xscale^2*obj.Yscale^2*p(:,3)^2)^(3/2);
            ddRdxdy =-(p(:,1)*obj.Xscale^2*p(:,2)*obj.Yscale^2*obj.Zscale^4)/(p(:,1)^2*obj.Yscale^2*obj.Zscale^2 + obj.Xscale^2*p(:,2)^2*obj.Zscale^2 + obj.Xscale^2*obj.Yscale^2*p(:,3)^2)^(3/2);
            ddRdxdz =-(p(:,1)*obj.Xscale^2*obj.Yscale^4*p(:,3)*obj.Zscale^2)/(p(:,1)^2*obj.Yscale^2*obj.Zscale^2 + obj.Xscale^2*p(:,2)^2*obj.Zscale^2 + obj.Xscale^2*obj.Yscale^2*p(:,3)^2)^(3/2);
            ddRdydx =-(p(:,1)*obj.Xscale^2*p(:,2)*obj.Yscale^2*obj.Zscale^4)/(p(:,1)^2*obj.Yscale^2*obj.Zscale^2 + obj.Xscale^2*p(:,2)^2*obj.Zscale^2 + obj.Xscale^2*obj.Yscale^2*p(:,3)^2)^(3/2);
            ddRdydy =(obj.Xscale^2*obj.Yscale^2*obj.Zscale^2*(p(:,1)^2*obj.Zscale^2 + obj.Xscale^2*p(:,3)^2))/(p(:,1)^2*obj.Yscale^2*obj.Zscale^2 + obj.Xscale^2*p(:,2)^2*obj.Zscale^2 + obj.Xscale^2*obj.Yscale^2*p(:,3)^2)^(3/2);
            ddRdydz =-(obj.Xscale^4*p(:,2)*obj.Yscale^2*p(:,3)*obj.Zscale^2)/(p(:,1)^2*obj.Yscale^2*obj.Zscale^2 + obj.Xscale^2*p(:,2)^2*obj.Zscale^2 + obj.Xscale^2*obj.Yscale^2*p(:,3)^2)^(3/2);
            ddRdzdx =-(p(:,1)*obj.Xscale^2*obj.Yscale^4*p(:,3)*obj.Zscale^2)/(p(:,1)^2*obj.Yscale^2*obj.Zscale^2 + obj.Xscale^2*p(:,2)^2*obj.Zscale^2 + obj.Xscale^2*obj.Yscale^2*p(:,3)^2)^(3/2);
            ddRdzdy =-(obj.Xscale^4*p(:,2)*obj.Yscale^2*p(:,3)*obj.Zscale^2)/(p(:,1)^2*obj.Yscale^2*obj.Zscale^2 + obj.Xscale^2*p(:,2)^2*obj.Zscale^2 + obj.Xscale^2*obj.Yscale^2*p(:,3)^2)^(3/2);
            ddRdzdz =(obj.Xscale^2*obj.Yscale^2*obj.Zscale^2*(p(:,1)^2*obj.Yscale^2 + obj.Xscale^2*p(:,2)^2))/(p(:,1)^2*obj.Yscale^2*obj.Zscale^2 + obj.Xscale^2*p(:,2)^2*obj.Zscale^2 + obj.Xscale^2*obj.Yscale^2*p(:,3)^2)^(3/2);
            
            SecondGrad = [ddRdxdx ddRdxdy ddRdxdz;
                          ddRdydx ddRdydy ddRdydz;
                          ddRdzdx ddRdzdy ddRdzdz];
        end
        
        function [DF] = dFelimActual(obj,p,slavedof)
            if slavedof == 1
                s = p(1,1); %x
                m1 = p(1,2);
                m2 = p(1,3);
                
                as = obj.Xscale;
                bm1 = obj.Yscale;
                bm2 = obj.Zscale;
            elseif slavedof == 2
                s = p(1,2); %y slave
                m1 = p(1,1);
                m2 = p(1,3);
                
                as = obj.Yscale;
                bm1 = obj.Xscale;
                bm2 = obj.Zscale;
            elseif slavedof == 3
                s = p(1,3); %z slave
                m1 = p(1,1);
                m2 = p(1,2);
                
                as = obj.Zscale;
                bm1 = obj.Xscale;
                bm2 = obj.Yscale;
            end
            
            dSdm1 =-(as^2*m1)/(bm1^2*s);
            dSdm2 =-(as^2*m2)/(bm2^2*s);

            DF = [dSdm1 dSdm2];
        end
        
        function [SecondGrad] = ddFelimActual(obj,p,slavedof)
            %ddFelimActual Second order actual derivitives for ellimination
            %  used for La Grange and CB2 Ellim approx
            
            if slavedof == 1
                s = p(1,1); %x slave
                m1 = p(1,2);
                m2 = p(1,3);
                
                as = obj.Xscale;
                bm1 = obj.Yscale;
                bm2 = obj.Zscale;
                
            elseif slavedof == 2
                s = p(1,2); %y slave
                m1 = p(1,1);
                m2 = p(1,3);
                
                as = obj.Yscale;
                bm1 = obj.Xscale;
                bm2 = obj.Zscale;
            elseif slavedof == 3
                
                s = p(1,3); %z slave
                m1 = p(1,1);
                m2 = p(1,2);
                
                as = obj.Zscale;
                bm1 = obj.Xscale;
                bm2 = obj.Yscale;
            end
            
            % ddS1d1d1 = -(as/bm1)^2*(s.^2 + (as/bm1)^2*m1.^2)/s^3;
            % ddS1d1d2 = (as^4/(bm1^2*bm2^2))*m1*m2/s^3;
            % ddS2d2d1 = (as^4/(bm1^2*bm2^2))*m1*m2/s^3;
            % ddS2d2d2 = -(as/bm1)^2*(s.^2 + (as/bm1)^2*m1.^2)/s^3;
            
            ddSdM1dM1 =-(as^4*(bm2^2 - m2^2))/(bm1^2*bm2^2*s^3);
            ddSdM1dM2 =-(as^4*m1*m2)/(bm1^2*bm2^2*s^3);
            ddSdM2dM1 =-(as^4*m1*m2)/(bm1^2*bm2^2*s^3);
            ddSdM2dM2 =-(as^4*(bm1^2 - m1^2))/(bm1^2*bm2^2*s^3);
            
            % SecondGrad = [ddS1d1d1 ddS1d1d2
            %               ddS2d2d1 ddS2d2d2];
            
            SecondGrad = [ddSdM1dM1 ddSdM1dM2
                          ddSdM2dM1 ddSdM2dM2];
        end
        
        function [pOut]= ClosestPoint(obj,p) %Gives point on closest boundary
            Np = size(p,1);
            pOut = zeros(size(p));
            for i = 1:Np
                fx = @(t) obj.Xscale*cos(2*pi*t);
                fy = @(t) obj.Zscale*sin(2*pi*t);
                
                f = @(x) (fx(x)-p(i,1))^2 + (fy(x)-p(i,2))^2;
                
                [t1, fval1] = fminsearch(f,0);
                [t2, fval2] = fminsearch(f,0.3);
                [t3, fval3] = fminsearch(f,0.6);
                
                if fval1 < fval2
                    t = t1;
                    fn = fval1;
                else
                    t = t2;
                    fn = fval2;
                end
                
                if fval3 < fn
                    t = t3;
                end
                
                pOut(i,:) = [fx(t) fy(t)];
            end
            
        end
        
        function [vecOut]= distance(obj,ps)
            %Gives vector to closest boundary point
            grads = obj.dF(ps);
            vecOut = -(grads./sqrt(sum(grads.^2,2))).*obj.fd(ps);
            
            
        end
        
        function [pOut]= ClosestPoint2(obj,p) %Gives point on closest boundary
            Np = size(p,1);
            pOut = zeros(size(p));
            fx = @(t) obj.Xscale*cos(2*pi*t);
            fy = @(t) obj.Zscale*sin(2*pi*t);
            
            dfx = @(t) -obj.Xscale*sin(2*pi*t)*2*pi;
            dfy = @(t) obj.Zscale*cos(2*pi*t)*2*pi;
            
            for i = 1:Np
                
                f = @(x) ((fx(x)-p(i,1))^2 + (fy(x)-p(i,2))^2)^0.5;
                
                len = norm(p);
                if (p(1,1) >= 0) && (p(1,2) >=0)
                    tstart = asin(p(1,2)/len)./2/pi;
                elseif (p(1,1) <= 0) && (p(1,2) >= 0)
                    tstart = 0.5 - asin(p(1,2)/len)./2/pi;
                elseif (p(1,1) <= 0) && (p(1,2) <=0)
                    tstart = 0.5 - asin(p(1,2)/len)./2/pi;
                else
                    tstart = 1 +  asin(p(1,2)/len)./2/pi;
                end
                
                t = fminsearch(f,tstart);
                
                pOut(i,:) = [fx(t) fy(t)];
            end
            
        end
        
        function [pOut]= ClosestPoint3(obj,p) %Gives point on closest boundary
            Np = size(p,1);
            pOut = zeros(size(p));
            fx = @(t) obj.Xscale*cos(2*pi*t);
            fy = @(t) obj.Zscale*sin(2*pi*t);
            
            dfx = @(t) -obj.Xscale*sin(2*pi*t)*2*pi;
            dfy = @(t) obj.Zscale*cos(2*pi*t)*2*pi;
            
            for i = 1:Np
                
                f = @(x) (fx(x)-p(i,1))^2 + (fy(x)-p(i,2))^2;
                g = @(x) (fx(x)-p(i,1))*2*dfx(x)* + (fy(x)-p(i,2))*2*dfy(x);
                fun = @(x) obj.myoptimfunction(f,g,x);
                len = norm(p);
                if (p(1,1) >= 0) && (p(1,2) >=0)
                    tstart = asin(p(1,2)/len)./2/pi;
                elseif (p(1,1) <= 0) && (p(1,2) >= 0)
                    tstart = 0.5 - asin(p(1,2)/len)./2/pi;
                elseif (p(1,1) <= 0) && (p(1,2) <=0)
                    tstart = 0.5 - asin(p(1,2)/len)./2/pi;
                else
                    tstart = 1 +  asin(p(1,2)/len)./2/pi;
                end
                
                t = fminsearch(fun,tstart);
                
                pOut(i,:) = [fx(t) fy(t)];
            end
            
        end
        
        
        function xy = plot(obj)
            %plot Gives boudary plot points
            % This returns signed distance function for a given point
            
            x(:,1) = linspace(-obj.Xscale,obj.Xscale,1000);
            y1 = (-(x./obj.Xscale).^2 + 1).^0.5.*obj.Zscale;
            y2 = -(-(x./obj.Xscale).^2 + 1).^0.5*obj.Zscale;
            xy =[x, y1;
                flip(x), flip(y2)];
        end
        
        function [Grad] = tangent(obj,p)
            %dF Summary of this method goes here
            %  This returns the tangent at the point
            
            dRdu1(:,1) = (2*p(:,1))./obj.Xscale^2;
            dRdu2(:,1) = (2*p(:,2))./obj.Zscale^2;
            lclGrad = [dRdu1 dRdu2];
            Grad  = lclGrad./norm(lclGrad);
        end
        
    end
    
    methods (Access = private)
        
        function [F, G] = myoptimfunction(obj,f,g,t)
            %dF Summary of this method goes here
            %  This returnd the first order partial derivitves of the function
            F = f(t);
            G = g(t);
            
            
        end
        
        
    end
end

