%MESHDEMO2d Distmesh2d examples.

%   Copyright (C) 2004-2012 Per-Olof Persson. See COPYRIGHT.TXT for details.
clear all; clc; close all;
rand('state',1); % Always the same results
set(gcf,'renderer','opengl');

fstats=@(p,t) fprintf('%d nodes, %d elements, min quality %.2f\n', ...
                      size(p,1),size(t,1),min(simpqual(p,t)));

fprintf('Uniform Mesh on Unit Circle\n');
% echo on

%% Normal run
% Cb2 FIXED Points on surface
tol = 10^-8;
format short
format compact
r=1;
h0 = .05;
n = round(2*pi*r/(.95*h0));
 if mod(n,2) ==1 % this is to ensure that a odd number of pfix nodes exist. otherwise prgram fails
       n = n+2;
 end   
t = (0:1/(n*r):1)'+.01; %plot 40 points along boundary
pfix = [cos(2*pi()*t(1:end-1)) sin(2*pi()*t(1:end-1))]*r;

pfix = [pfix(:,2) pfix(:,1)];
fd=@(p) sqrt(sum(p.^2,2))-r;

legends = {};
cols = {};
cols{size(cols,2)+1} = 'r';
legends{size(legends,2)+1} ='Ftruss = \DeltaL/L_d_e_s';

cols{size(cols,2)+1} = 'b';
legends{size(legends,2)+1} = 'Ftruss = \DeltaL/L_c';

S1 = 0.5;
cols{size(cols,2)+1} = 'k'
legends{size(legends,2)+1} = ['Ftruss = \DeltaL/(L_d_e_s)\timesexp(S_1[\DeltaL/L_d_e_s]^2) : S_1 = ' num2str(S1)];

S1 = 1;
cols{size(cols,2)+1} = 'k:'
legends{size(legends,2)+1} = ['Ftruss = \DeltaL/L_d_e_s\timesexp(S_1[\DeltaL/L_d_e_s]^2) : S_1 = ' num2str(S1)];

S1 = 2;
cols{size(cols,2)+1} = 'k--'
legends{size(legends,2)+1} = ['Ftruss = \DeltaL/L_d_e_s\timesexp(S_1[\DeltaL/L_d_e_s]^2) : S_1 = ' num2str(S1)];

S1 = -1;
cols{size(cols,2)+1} = 'k-.'
legends{size(legends,2)+1} = ['Ftruss = \DeltaL/L_d_e_s \times exp(S_1[\DeltaL/L_d_e_s]^2) : S_1 = ' num2str(S1)];

S1 = 1;
cols{size(cols,2)+1} = 'g'
legends{size(legends,2)+1} = ['Ftruss = \DeltaL/L_c \times exp(S_1[\DeltaL/L_c]^2) : S_1 = ' num2str(S1)];
 

types = [2.0  1.0
         1.0  1.0
         4.0  0.5
         4.0  1.0
         4.0  2.0
         4.0  -0.5
         3.0  1.0];

scale = 5;

results = inf;
MeshedAtIter = 0;

cols{size(cols,2)+1} = 'g'
legends{size(legends,2)+1} = 'Remeshes';
remeshes = zeros(0,2);
for i = 1:size(types,1)
[p,t, iters, updateNorms,AverageLcLdes,MeshedAtIter] = CB_distmesh2d(fd,@(p) huniformScale(p,h0,scale),h0,r.*[-1,-1;1,1],pfix,tol,types(i,1), types(i,2));
minqual = min(simpqual(p,t))

AllUpdates{i}=updateNorms;
% figure(i)
% cla,patch('vertices',p,'faces',t,'edgecol','k','facecol',[.8,.9,1]);
% drawnow
updateNorms2 = [updateNorms; 5*ones(100,1)];
MeshedAtIter = [MeshedAtIter; zeros(100,1)];
S1 = types(i,2)
if results == inf
    results =      [i;S1;iters; AverageLcLdes; minqual; sum(MeshedAtIter);0;updateNorms2(1:18,1)];
else
    results(:,i) = [i;S1;iters; AverageLcLdes; minqual; sum(MeshedAtIter);0;updateNorms2(1:18,1)];
end

figure(10)
hold on
plot(1:size(updateNorms,1),log10(updateNorms),cols{i})
drawnow

remeshes = [remeshes; find(MeshedAtIter ==1) updateNorms2(find(MeshedAtIter ==1))];
figure(20+i)
    cla,patch('vertices',p,'faces',t,'edgecol','k','facecol',[.8,.9,1]);
    hold on
    plot(pfix(:,1),pfix(:,2),'k*');
    drawnow
end
figure(10)
hold on
plot(remeshes(:,1),log10(remeshes(:,2)),'k.')
legend(legends)
title({'Line search','Truss Stiffness Formula : Convergence',['Scale = L_c/L_d_e_s = ' num2str(scale)],'Start search : \alpha = 0'},'FontSize',16)
xlabel('Iter','FontSize',12)
ylabel('log(norm(err))','FontSize',12)
axis auto
dim = [.2 .5 .3 .3];
annotation('textbox',dim,'String','\DeltaL = L_d_e_s - L_c','FitBoxToText','on');



format long
results