classdef EllipseFunction1
    %FUNCLASS Summary of this class goes here
    %   Detailed explanation goes here
    
    properties 
        Height = .1;
        Width = 1;
        Grad
    end
    
    methods (Access = public)
        
        function [Func] = Func(obj,p)
            %Func Summary of this method goes here
            % This returns signed distance function for a given point
            Func = (p(:,1)./obj.Width)^2 + (p(:,2)./obj.Height)^2 - 1;
        end
        
        function [Grad] = dF(obj,p)
            %dF Summary of this method goes here
            %  This returnd the first order partial derivitves of the function
            dRdu1(:,1) = (2*p(:,1))./obj.Width^2;
            dRdu2(:,1) = (2*p(:,2))./obj.Height^2;
            lclGrad = [dRdu1 dRdu2];
            Grad  = lclGrad./norm(lclGrad);
        end
        
        function [SecondGrad] = ddF(p)
            %METHOD1 Summary of this method goes here
            %   Detailed explanation goes here
            
            ddRdu1du1 = 2/obj.Width^2;
            ddRdu1du2 = 0;
            ddRdu2du1 = 0;
            ddRdu2du2 = 2/obj.Height^2;
            
            SecondGrad = [ddRdu1du1 ddRdu1du2;
                          ddRdu2du1 ddRdu2du2];
        end
        
        function [vecOut]= distance(obj,p) 
            %Gives vector to closest boundary
            pboundary = ClosestPoint(p);
            vecOut = pboundary-p;
        end
        
        function [pOut]= ClosestPoint(obj,p) %Gives point on closest boundary
            fx = @(t) obj.Width*cos(2*pi*t);
            fy = @(t) obj.Height*sin(2*pi*t);
            
            f = @(x) (fx(x)-p(1,1))^2 + (fy(x)-p(1,2))^2;
            
            t1 = fminsearch(f,0);
            
            pOut = [fx(t1) fy(t1)];
            
        end
        
        function [pOut]= RoughlyClosestPoint(obj,p) %Gives vector to closest boundary
            
            gr = obj.dF(p);
            m = gr(1,2)/gr(1,1);
            c = p(1,2) - m*p(1,1);
            
            W = obj.Width;
            H = obj.Height;
            
            x1 =  (W*(H*(H^2 + W^2*m^2 - c^2)^(1/2) - W*c*m))/(H^2 + W^2*m^2);
            x2 = -(W*(H*(H^2 + W^2*m^2 - c^2)^(1/2) + W*c*m))/(H^2 + W^2*m^2);
                        
            y1 =  sqrt(1-(x1./obj.Width)^2)*obj.Height;
            y2 =  sqrt(1-(x2./obj.Width)^2)*obj.Height;
            y3 = -sqrt(1-(x1./obj.Width)^2)*obj.Height;
            y4 = -sqrt(1-(x2./obj.Width)^2)*obj.Height;
            
            pOut = [x1 y1
                    x2 y2
                    x1 y3
                    x2 y4];
        end
                
        function [pOut]= ClosestPoint2(obj,p) %Gives vector to closest boundary
            
            F = @(x) obj.myoptimfunction(x,p);
            F(p)    
            x0 = p;
            X = fsolve(@(x) F(x),x0);
            
            pOut = X;
        
        end
        
        function xy = plot(obj)
            %Func Summary of this method goes here
            % This returns signed distance function for a given point
          
            x(:,1) = linspace(-obj.Width,obj.Width,1000);
            y1 = (-(x./obj.Width).^2 + 1).^0.5.*obj.Height;
            y2 = -(-(x./obj.Width).^2 + 1).^0.5*obj.Height;
            xy =[x, y1;
                 flip(x), flip(y2)];   
        end
        
        function [Grad] = tangent(obj,p)
            %dF Summary of this method goes here
            %  This returns the first order partial derivitves of the function
            dRdu1(:,1) = (2*p(:,1))./obj.Width^2;
            dRdu2(:,1) = (2*p(:,2))./obj.Height^2;
            lclGrad = [dRdu1 dRdu2];
            Grad  = lclGrad./norm(lclGrad);
        end

    end
    
    methods (Access = private)
                 
         function F = myoptimfunction(obj,x,p)
            %dF Summary of this method goes here
            %  This returnd the first order partial derivitves of the function
            F(1) = (2*x(2)/(obj.Height)^2)*(p(1,1) - x(1)) - (p(1,2) - x(2))*(2*x(1)/(obj.Width)^2);
            F(2) = (x(2)/obj.Height)^2 + (x(1)/obj.Width)^2  -1;
         end
    end
end

