classdef EllipseFunction5_3d
    %FUNCLASS Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        Xscale = 1; %x
        Yscale = 2;%y
        Zscale = 2; %z
    end
    
    methods (Access = public)
        
        function pfixout = pfix(obj)
            pfixout =[obj.Xscale 0 0
                -obj.Xscale 0 0
                0 obj.Yscale 0
                0 -obj.Yscale 0
                0 0 obj.Zscale
                0 0 -obj.Zscale];
        end
        
        function pboundsout = pbounds(obj)
            pboundsout =[-obj.Xscale -obj.Yscale -obj.Zscale
                obj.Xscale  obj.Yscale  obj.Zscale];
        end
        
        function [Func] = fd(obj,p)
            %Func This returns the distance function
            % This returns signed distance function for a given point
            
            %             Func = ((p(:,1)./obj.Xscale).^2 + (p(:,2)./obj.Yscale).^2 + (p(:,3)./obj.Zscale).^2).^0.5 - 1;
            Func = sign(((p(:,1)./obj.Xscale).^2 + (p(:,2)./obj.Yscale).^2 + (p(:,3)./obj.Zscale).^2).^0.5 - 1).*(sum(obj.distance(p).^2,2)).^0.5;
        end
        
        function [Grad] = dF(obj,p)
            %dF Summary of this method goes here
            %  This returnd the first order partial derivitves of the
            %  function for a given point
            dRdu1(:,1) = (p(:,1)./obj.Xscale.^2)./((p(:,1)./obj.Xscale).^2 + (p(:,2)./obj.Yscale).^2 + (p(:,3)./obj.Zscale).^2).^0.5;
            dRdu2(:,1) = (p(:,2)./obj.Yscale.^2)./((p(:,1)./obj.Xscale).^2 + (p(:,2)./obj.Yscale).^2 + (p(:,3)./obj.Zscale).^2).^0.5;
            dRdu3(:,1) = (p(:,3)./obj.Zscale.^2)./((p(:,1)./obj.Xscale).^2 + (p(:,2)./obj.Yscale).^2 + (p(:,3)./obj.Zscale).^2).^0.5;
            Grad = [dRdu1 dRdu2 dRdu3];
        end
        
        function [SecondGrad] = ddF(obj,p)
            %ddF Second order partial derivitives (risidual derivitives)
            %  used for La Grange and CB2 Ellim approx
            
            ddRdxdx =-(p(1,1).^2 - obj.Xscale.^2.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2))./(obj.Xscale.^4.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2).^(3./2));
            ddRdxdy =-(p(1,1).*p(1,2))./(obj.Xscale.^2.*obj.Yscale.^2.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2).^(3./2));
            ddRdxdz =-(p(1,1).*p(1,3))./(obj.Xscale.^2.*obj.Zscale.^2.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2).^(3./2));
            ddRdydx =-(p(1,1).*p(1,2))./(obj.Xscale.^2.*obj.Yscale.^2.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2).^(3./2));
            ddRdydy =-(p(1,2).^2 - obj.Yscale.^2.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2))./(obj.Yscale.^4.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2).^(3./2));
            ddRdydz =-(p(1,2).*p(1,3))./(obj.Yscale.^2.*obj.Zscale.^2.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2).^(3./2));
            ddRdzdx =-(p(1,1).*p(1,3))./(obj.Xscale.^2.*obj.Zscale.^2.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2).^(3./2));
            ddRdzdy =-(p(1,2).*p(1,3))./(obj.Yscale.^2.*obj.Zscale.^2.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2).^(3./2));
            ddRdzdz =-(p(1,3).^2 - obj.Zscale.^2.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2))./(obj.Zscale.^4.*(p(1,1).^2./obj.Xscale.^2 + p(1,2).^2./obj.Yscale.^2 + p(1,3).^2./obj.Zscale.^2).^(3./2));
            
            SecondGrad = [ddRdxdx ddRdxdy ddRdxdz;
                          ddRdydx ddRdydy ddRdydz;
                          ddRdzdx ddRdzdy ddRdzdz];
        end
        
        function [DF] = dFelimActual(obj,p,slavedof)
            if slavedof == 1
                s = p(1,1); %x
                m1 = p(1,2);
                m2 = p(1,3);
                
                as = obj.Xscale;
                bm1 = obj.Yscale;
                bm2 = obj.Zscale;
            elseif slavedof == 2
                s = p(1,2); %y slave
                m1 = p(1,1);
                m2 = p(1,3);
                
                as = obj.Yscale;
                bm1 = obj.Xscale;
                bm2 = obj.Zscale;
            elseif slavedof == 3
                s = p(1,3); %z slave
                m1 = p(1,1);
                m2 = p(1,2);
                
                as = obj.Zscale;
                bm1 = obj.Xscale;
                bm2 = obj.Yscale;
            end
            
            dSdm1 =-(as^2*m1)/(bm1^2*s);
            dSdm2 =-(as^2*m2)/(bm2^2*s);

            DF = [dSdm1 dSdm2];
        end
        
        function [SecondGrad] = ddFelimActual(obj,p,slavedof)
            %ddFelimActual Second order actual derivitives for ellimination
            %  used for La Grange and CB2 Ellim approx
            
            if slavedof == 1
                s = p(1,1); %x slave
                m1 = p(1,2);
                m2 = p(1,3);
                
                as = obj.Xscale;
                bm1 = obj.Yscale;
                bm2 = obj.Zscale;
                
            elseif slavedof == 2
                s = p(1,2); %y slave
                m1 = p(1,1);
                m2 = p(1,3);
                
                as = obj.Yscale;
                bm1 = obj.Xscale;
                bm2 = obj.Zscale;
            elseif slavedof == 3
                
                s = p(1,3); %z slave
                m1 = p(1,1);
                m2 = p(1,2);
                
                as = obj.Zscale;
                bm1 = obj.Xscale;
                bm2 = obj.Yscale;
            end
            
            % ddS1d1d1 = -(as/bm1)^2*(s.^2 + (as/bm1)^2*m1.^2)/s^3;
            % ddS1d1d2 = (as^4/(bm1^2*bm2^2))*m1*m2/s^3;
            % ddS2d2d1 = (as^4/(bm1^2*bm2^2))*m1*m2/s^3;
            % ddS2d2d2 = -(as/bm1)^2*(s.^2 + (as/bm1)^2*m1.^2)/s^3;
            
            ddSdM1dM1 =-(as^4*(bm2^2 - m2^2))/(bm1^2*bm2^2*s^3);
            ddSdM1dM2 =-(as^4*m1*m2)/(bm1^2*bm2^2*s^3);
            ddSdM2dM1 =-(as^4*m1*m2)/(bm1^2*bm2^2*s^3);
            ddSdM2dM2 =-(as^4*(bm1^2 - m1^2))/(bm1^2*bm2^2*s^3);
            
            % SecondGrad = [ddS1d1d1 ddS1d1d2
            %               ddS2d2d1 ddS2d2d2];
            
            SecondGrad = [ddSdM1dM1 ddSdM1dM2
                          ddSdM2dM1 ddSdM2dM2];
        end
        
        function [pOut]= ClosestPoint(obj,p) %Gives point on closest boundary
            Np = size(p,1);
            pOut = zeros(size(p));
            
            
            for i = 1:Np
                
%                 f = @(x) 1*F1(x')^2 + 1*sum((p(i,:)-x').^2);
                f = @(x) obj.myoptimfunction(p(i,:),x);

                xstart = p(i,:);
                options = optimoptions(@fminunc,'Algorithm','trust-region',...
                    'SpecifyObjectiveGradient',true,'HessianFcn','objective');
                [x,fval,exitflag,output] = fminunc(f,xstart,options);
                xout = fminsearch(f,p(i,:)');
                
                pOut(i,:) = xout';
            end
            
        end
        
        function [vecOut]= distance(obj,ps)
            %Gives vector to closest boundary point
            
            pboundary = obj.ClosestPoint2(ps);
            vecOut = pboundary - ps;
            
        end
        
        function [pOut]= ClosestPoint2(obj,p) %Gives point on closest boundary
            Np = size(p,1);
            pOut = zeros(size(p));
            F1 = @(p) ((p(:,1)./obj.Xscale).^2 + (p(:,2)./obj.Yscale).^2 + (p(:,3)./obj.Zscale).^2)- 1;
            
            for i = 1:Np
                
                f = @(x) 100*F1(x')^2 + sum((p(i,:)-x').^2);
                
%                 if (p(1,1) >= 0) && (p(1,2) >=0)
%                     tstart = asin(p(1,2)/len)./2/pi;
%                 elseif (p(1,1) <= 0) && (p(1,2) >= 0)
%                     tstart = 0.5 - asin(p(1,2)/len)./2/pi;
%                 elseif (p(1,1) <= 0) && (p(1,2) <=0)
%                     tstart = 0.5 - asin(p(1,2)/len)./2/pi;
%                 else
%                     tstart = 1 +  asin(p(1,2)/len)./2/pi;
%                 end
                               
                xout = fminsearch(f,p(i,:)');
                
                pOut(i,:) = xout';
            end
            
        end
        
               
        function xy = plot(obj)
            %plot Gives boudary plot points
            % This returns signed distance function for a given point
            
            x(:,1) = linspace(-obj.Xscale,obj.Xscale,1000);
            y1 = (-(x./obj.Xscale).^2 + 1).^0.5.*obj.Zscale;
            y2 = -(-(x./obj.Xscale).^2 + 1).^0.5*obj.Zscale;
            xy =[x, y1;
                flip(x), flip(y2)];
        end
        
        function [Grad] = tangent(obj,p)
            %dF Summary of this method goes here
            %  This returns the tangent at the point
            
            dRdu1(:,1) = (2*p(:,1))./obj.Xscale^2;
            dRdu2(:,1) = (2*p(:,2))./obj.Zscale^2;
            lclGrad = [dRdu1 dRdu2];
            Grad  = lclGrad./norm(lclGrad);
        end
        
    end
    
    methods (Access = public)
        
        function [F, G, H] = myoptimfunction(obj,p,x)
            %dF Summary of this method goes here
            %  This returnd the first order partial derivitves of the function
            
            F1 = @(p) ((p(:,1)./obj.Xscale).^2 + (p(:,2)./obj.Yscale).^2 + (p(:,3)./obj.Zscale).^2)- 1;
            F2 = @(x,p) (x(1) - p(1,1))^2 + (x(2) - p(1,2))^2 + (x(3) - p(1,3))^2;
            G2 = @(x,p) [2*(x(1) - p(1,1)),  2*(x(2) - p(1,2)), 2*(x(3) - p(1,3))];
            H2 = @(x,p) [2 0 0; 0 2 0; 0 0 2];
            
            f = @(x) 1*F1(x')^2 + F2(x,p);
            g = @(x) obj.dF(x')' + G2(x',p)';
            h = @(x) obj.ddF(x')' + H2(x',p)';
            
            F = f(x);
            G = g(x);
            H = h(x);
            
        end
    end
end

